@import 'MochaJSDelegate.js'
@import 'utils.js'

function getLinkAttributeString(text, linkStr, isHighLight) {
  var str = [[NSString alloc] initWithString: text];
  var color = isHighLight ? [NSColor blueColor] : [NSColor grayColor];
  var string = [[NSMutableAttributedString new] initWithString: str]; // assume string exists
  var urlRange = [str rangeOfString:linkStr];
  var url = [NSURL URLWithString:"http://www.baidu.com"];
  [string addAttribute:NSLinkAttributeName
                   value:url
                   range:urlRange];
  [string addAttribute:NSForegroundColorAttributeName
                   value:color
                   range:urlRange];
  [string addAttribute:NSUnderlineStyleAttributeName
                  value:[NSNumber numberWithInteger:NSUnderlineStyleSingle]
                  range:urlRange];
  return string;
}

function getLinkButton(text, linkStr, rect, clickCallback) {
  var btn = [[NSButton alloc] initWithFrame:rect];
  [btn setBordered:false];
  
  [btn setAttributedTitle: getLinkAttributeString(text, linkStr, false)];
  [btn setCOSJSTargetFunction:function(sender) {
    clickCallback();
  }];
  [btn setAction:"callAction:"];
  var  delegate = new MochaJSDelegate({
    "mouseEntered:":function(theEvent) {
      [btn setAttributedTitle: getLinkAttributeString(text, linkStr, true)];
    },
    "mouseExited:":function(theEvent) {
      [btn setAttributedTitle: getLinkAttributeString(text, linkStr, false)];
    },
  });
  var trackingArea =[[NSTrackingArea alloc] initWithRect:[btn bounds] 
                                    options:NSTrackingMouseEnteredAndExited|NSTrackingActiveInKeyWindow 
                                    owner:delegate.getClassInstance() 
                                    userInfo:nil];
  [btn addTrackingArea:trackingArea];

  return btn;
}

function getCenterLogo(width, y, size) {
  
  var x = (width - size) / 2;
  var imageView = [[NSImageView alloc] initWithFrame: NSMakeRect(x, y, size, size)];
  var file = sketchApp.resourceNamed("logo.png");
  var icon = NSImage.alloc().initByReferencingURL(file);
  [imageView setImage: icon];
  return imageView;
}

function getCenterTitle(width, y, h, text) {
  var fontManager = [NSFontManager sharedFontManager];
  var titleField = [[NSTextField alloc] initWithFrame:NSMakeRect(0, y, 200, h)];
  [titleField setEditable:false];
  [titleField setBordered:false];
  [titleField setDrawsBackground:false];
  var titleFont =[fontManager fontWithFamily:"Helvetica" traits:NSBoldFontMask weight:7 size:15];
  [titleField setFont:titleFont];
  [titleField setStringValue:text];
  var titleWidth = titleField.fittingSize().width;
  titleField.frame = NSMakeRect((width-titleWidth)/2, y,titleWidth, h);
  return titleField;
}

function getCenterSubtitle(width, y, h, text) {
  var fontManager = [NSFontManager sharedFontManager];
  var subtitleField = [[NSTextField alloc] initWithFrame:NSMakeRect(0, y, 550, h)];
  [subtitleField setEditable:false];
  [subtitleField setBordered:false];
  var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
  [subtitleField setFont:subtitleFont];
  [subtitleField setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
  [subtitleField setDrawsBackground:false];
  [subtitleField setStringValue:text];
  var subtitleWidth = subtitleField.fittingSize().width;
  subtitleField.frame = NSMakeRect((width-subtitleWidth)/2, y,subtitleWidth, h);
  return subtitleField;
}

function getInputText(rect, text, placeHolder, isPassword) {
  var inputField;
  if(isPassword) {
    inputField = [[NSSecureTextField alloc] initWithFrame:rect];
  } else {
    inputField = [[NSTextField alloc] initWithFrame:rect];
  }
  [inputField setEditable:true];
  [inputField setDrawsBackground:false];
  [inputField setStringValue:text]
  [inputField setFont:[NSFont systemFontOfSize:13]];
  [inputField setPlaceholderString:placeHolder] 
  [inputField setWantsLayer: true];
  [[inputField layer] setBorderColor:[NSColor redColor].CGColor];
  [[inputField layer] setBorderWidth:1];
  return inputField;
}

function getScrollView(rect, documentView) {
  var scrollView = [[NSScrollView alloc] initWithFrame:rect];
  [scrollView setHasVerticalScroller:true];
  [scrollView setHasHorizontalScroller:true];
  [scrollView setBorderType:NSNoBorder];
  [scrollView setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
  [scrollView setDrawsBackground:false];
  [scrollView setAutohidesScrollers:true];
  [[scrollView verticalScroller] setScrollerStyle:NSScrollerStyleLegacy];
  [scrollView setDocumentView:documentView];
  [[scrollView contentView] scrollToPoint:NSMakePoint(0, NSMaxY([[scrollView documentView] frame]) - [[scrollView contentView] bounds].size.height)];
  [scrollView reflectScrolledClipView: [scrollView contentView]];
  return scrollView;
}

var projectComboFragment = {
  deviceTypeCombo: null,
  orientationCombo: null,

  getProjectCombo: function(rect) {
    var projectComboView = [[NSView alloc] initWithFrame:rect];
    projectComboView.setWantsLayer(true);

    var deviceTypeLabel =[[NSTextField alloc] initWithFrame: NSMakeRect(0, 75, 300, 25)];
    [deviceTypeLabel setEditable:false];
    [deviceTypeLabel setBordered:false];
    [deviceTypeLabel setFont:[NSFont systemFontOfSize:13]];
    [deviceTypeLabel setDrawsBackground:false];
    [deviceTypeLabel setStringValue:text.select_project_type];
    [projectComboView addSubview:deviceTypeLabel];

    var deviceTypeCombo = this.createDeviceTypeCombo(NSMakeRect(0, 42, 210, 25));
    [projectComboView addSubview:deviceTypeCombo];

    var orientationCombo = this.createOrientationCombo(NSMakeRect(0, 5, 120, 25), deviceTypeCombo);
    [projectComboView addSubview:orientationCombo];

    [deviceTypeCombo setCOSJSTargetFunction:function(sender) {
      var projectIndex = [deviceTypeCombo indexOfSelectedItem];
      if([0,1].indexOf(projectIndex) === -1)
        [orientationCombo setEnabled: false];
      else
        [orientationCombo setEnabled: true];
    }];

    return projectComboView;
  },

  getDeviceTypeCombo: function() {
    return this.deviceTypeCombo;
  },

  getOrientationCombo: function() {
    return this.orientationCombo;
  },

  createDeviceTypeCombo: function(rect) {
    var deviceTypeCombo = [[NSPopUpButton alloc] initWithFrame: rect];
    deviceTypeCombo.addItemsWithTitles(deviceTypesStr);
    [deviceTypeCombo setFont:[NSFont systemFontOfSize:13]];
    [deviceTypeCombo selectItemAtIndex:0];
    var defaultDevice = getDefault("exportDevice",null);
    if(defaultDevice){
      var indexDevice = deviceTypes.indexOf(defaultDevice.trim());
      if(indexDevice>=0)
        [deviceTypeCombo selectItemAtIndex:indexDevice];
    }
    [deviceTypeCombo setEnabled:false];
    this.deviceTypeCombo = deviceTypeCombo;
    return deviceTypeCombo;
  },

  createOrientationCombo: function(rect, deviceTypeCombo) {
    var orientationCombo = [[NSPopUpButton alloc] initWithFrame: rect];
    [orientationCombo setEnabled: ([0,1].indexOf([deviceTypeCombo indexOfSelectedItem]) !== -1)];
    [orientationCombo setFont:[NSFont systemFontOfSize:13]];
    orientationCombo.addItemsWithTitles(orientationStr);
    [orientationCombo selectItemAtIndex:0];
    var defaultOrientation = getDefault("exportOrientation",null);
    if(defaultOrientation){
      var indexOrientation = orientation.indexOf(defaultOrientation.trim());
      if(indexOrientation>=0)
        [orientationCombo selectItemAtIndex:indexOrientation];
    }
    this.orientationCombo = orientationCombo;
    return orientationCombo;
  }

}

var artboardScrollFragment = {

  mapPreviews: {},
  mapCheckboxes: {},

  createContainer: function() {
    var skArtboards = getAllArtboards(sketchDoc);
    var artboardsCount = skArtboards.length;

    var entryWidth = 92;
    var entryHeight = 98;
    var previewWidth = 76;
    var previewHeight = 76;

    var horizontalSpacing = 22;
    var verticalSpacing = 18;
    var rows = Math.ceil(artboardsCount/4);
    var containerHeight = Math.max(250,(rows*entryHeight)+((rows-1)*verticalSpacing));
    var artboardsContainer = [[NSView alloc] initWithFrame:NSMakeRect(0, 0, (entryWidth * 4)+(horizontalSpacing*3), containerHeight)];
    for (var i=0;i<artboardsCount;i++){
        var skArtboard=skArtboards[i];
        var name = skArtboard.name;

        var r = i % 4;
        var x = r * (entryWidth+horizontalSpacing);
        var y = containerHeight - (entryHeight) - (Math.floor(i/4)*(entryHeight+verticalSpacing));
        var artboard = [[NSView alloc] initWithFrame:NSMakeRect(x,y, entryWidth,entryHeight)];
        [artboardsContainer addSubview:artboard];

        var artboardView = [[NSImageView alloc] initWithFrame: NSMakeRect(8,20, previewWidth, previewHeight)];
        [artboard addSubview:artboardView];
        artboardView.setWantsLayer(true);
        [artboardView setToolTip:name];
        [[artboardView layer] setShadowOpacity:0.5];

        var artboardName =[[NSTextField alloc] initWithFrame: NSMakeRect(0, 2,72, 12)];

        [artboardName setEditable:false];
        [artboardName setBordered:false];
        [artboardName setFont:[NSFont systemFontOfSize:10]];
        [artboardName setDrawsBackground:false];
        [artboardName setStringValue:name];
        [artboardName setToolTip:name];
        var nameWidth = artboardName.fittingSize().width;
        var nameX = (entryWidth - nameWidth)/2;
        if(nameWidth>(entryWidth-15)){
          nameWidth = 72;
          nameX = 0;
        }else if(nameWidth>(entryWidth - (15*2)))
          nameX =  entryWidth - nameWidth - 15;
        artboardName.frame = NSMakeRect(nameX, 2,nameWidth, 12);

        [[artboardName cell] setLineBreakMode:NSLineBreakByTruncatingTail];
        [artboard addSubview:artboardName];

        artboardName.alignment = NSTextAlignmentCenter;

        var artboardCheck = [[NSButton alloc] initWithFrame: NSMakeRect(77,0, 16, 16)];
        [artboardCheck setButtonType:NSSwitchButton];
        [artboardCheck setTitle:@""];

        var contains = selectedArtboards.length==0 ? true :(selectedArtboards.indexOf(skArtboard.sketchObject)!=-1);
        [artboardCheck setState: (contains ? NSOnState : NSOffState)];

        [artboardCheck setEnabled:false];
        [artboard addSubview:artboardCheck];

        this.mapPreviews[i] = artboardView;
        this.mapCheckboxes[i] = artboardCheck;
    }
    return artboardsContainer;
  },

  getMapCheckBoxes: function() {
    return this.mapCheckboxes;
  },

  getMapPreviews: function() {
    return this.mapPreviews;
  }
}