@import 'utils.js'
@import 'MochaJSDelegate.js'
@import 'dialogHelper.js'

function openExportDialog(callback){
  var app = [NSApplication sharedApplication];
  var exportWindow = [[NSWindow alloc] init];
  var horizontalMargin = 35;
  var width = 770;
  var height = 550;

  COScript.currentCOScript().setShouldKeepAround_(true);
  [exportWindow setFrame:NSMakeRect(0, 0, width, height+20) display: false];
  var contentView = [exportWindow contentView];
  contentView.setWantsLayer(true);
  [contentView setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(235/255, 235/255, 235/255, 0.7)];

  var headerView = [[NSView alloc] initWithFrame:NSMakeRect(0, height-146, width, 146)];
  headerView.setWantsLayer(true);
  [headerView setBackgroundColor:NSColor.whiteColor()];
  [[exportWindow contentView] addSubview:headerView];

  
  var imageView = getCenterLogo(width, 76, 50);
  [headerView addSubview:imageView];

  var titleField = getCenterTitle(width, 38, 25, text.export_title);
  [headerView addSubview:titleField];

  var subtitleField = getCenterSubtitle(width, 12, 25, text.export_subtitle);
  [headerView addSubview:subtitleField];

  var skArtboards = getAllArtboards(sketchDoc);
  var artboardsCount = skArtboards.length;
  var previewWidth = 76;
  var previewHeight = 76;
  
  var container = artboardScrollFragment.createContainer();
  var scrollArtboards = getScrollView(NSMakeRect(20, height-146-250-35, 464, 250), container);
  [[exportWindow contentView] addSubview:scrollArtboards];
  var mapCheckboxes = artboardScrollFragment.getMapCheckBoxes();
  var mapPreviews = artboardScrollFragment.getMapPreviews();

  var exportAll =[[NSButton alloc] initWithFrame: NSMakeRect(25, 80, 120, 20)];
  [exportAll setButtonType:NSSwitchButton];
  [exportAll setTitle:text.export_all];
  [exportAll setFont:[NSFont systemFontOfSize:13]];
  var allselected = selectedArtboards.length==0 || selectedArtboards.length==artboardsCount;
  [exportAll setState:(allselected ? NSOnState : NSOffState)];
  [exportAll setEnabled:false];
  [[exportWindow contentView] addSubview:exportAll];
  [exportAll setCOSJSTargetFunction:function(sender) {
    for(var i=0;i<artboardsCount;i++){
      var artboardCheckbox = mapCheckboxes[i];
      [artboardCheckbox setState:[exportAll state]];
    }
  }];

  var borderVertical = [[NSView alloc] initWithFrame:NSMakeRect(499, height-146-316,1, 282)];
  [[exportWindow contentView] addSubview:borderVertical];
  borderVertical.setWantsLayer(true);
  [borderVertical setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];

  var projectComboView = projectComboFragment.getProjectCombo(NSMakeRect(514, 270, 300, 100));
  var deviceTypeCombo = projectComboFragment.getDeviceTypeCombo();
  var orientationCombo = projectComboFragment.getOrientationCombo();
  [[exportWindow contentView] addSubview:projectComboView];

  var export2x =[[NSButton alloc] initWithFrame: NSMakeRect(514, 216, 300, 25)];
  [export2x setButtonType:NSSwitchButton];
  [export2x setTitle: text.export_2X];
  [export2x setFont:[NSFont systemFontOfSize:13]];
  if(getDefault("export2x",0)==1)
    [export2x setState:NSOnState];
  [export2x setEnabled:false];
  [[exportWindow contentView] addSubview:export2x];

  var exportButton = [[NSButton alloc] initWithFrame:NSMakeRect(397, 15, 92, 30)],
  cancelButton = [[NSButton alloc] initWithFrame:NSMakeRect(280, 15, 92, 30)];
  [exportButton setTitle:text.export];
  [exportButton setBezelStyle:NSRoundedBezelStyle];
  [exportButton setAction:"callAction:"];
  [cancelButton setTitle:text.cancel];
  [cancelButton setBezelStyle:NSRoundedBezelStyle];
  [cancelButton setAction:"callAction:"];
  [exportButton setEnabled: true];
  [cancelButton setEnabled: true];

  var bottomActionsView = [[NSView alloc] initWithFrame:NSMakeRect(0, 0,width, 68)];
  [[exportWindow contentView] addSubview:bottomActionsView];
  var border = [[NSView alloc] initWithFrame:NSMakeRect(27, 67,width-54, 1)];
  [bottomActionsView addSubview:border];
  border.setWantsLayer(true);
  [border setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];


  var loadingView = [[NSView alloc] initWithFrame:NSMakeRect(0, 0, width, height-146)];
  loadingView.setWantsLayer(true);
  [loadingView setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(255/255, 255/255, 255/255, 0.7)];
  [[exportWindow contentView] addSubview:loadingView positioned:NSWindowAbove relativeTo:null];

  var activityIndicator = [[NSProgressIndicator alloc]initWithFrame:NSMakeRect(190,245,width-320,NSProgressIndicatorPreferredLargeThickness)];
  [activityIndicator setStyle:NSProgress​Indicator​Bar​Style];
  [activityIndicator setBezeled:true];
  [activityIndicator setMinValue:0];
  [activityIndicator setMaxValue:artboardsCount];
  [activityIndicator setDoubleValue:0];
  [activityIndicator setIndeterminate:false];
  [loadingView addSubview:activityIndicator];

  var result = {option: "", extra: {}};
  var dialogShutDown = function() {
    [exportWindow orderOut:nil];
    [[app mainWindow] endSheet: exportWindow];
    [exportButton setCOSJSTargetFunction:undefined];
    [cancelButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  };
  [exportButton setCOSJSTargetFunction:function(sender) {
    artboardsToExport = [];
    for(var i=0;i<artboardsCount;i++){
      var artboardCheckbox = mapCheckboxes[i];
      if([artboardCheckbox state]==NSOnState){
        artboardsToExport.push(skArtboards[i]);
      }
    }
    if(artboardsToExport.length==0){
        alert(text.uncheck_artboard_title, text.uncheck_artboard_content);
      return;
    }
    
    enableExportImage2X = [export2x state] == NSOnState;
    selectedDeviceType = deviceTypes[[deviceTypeCombo indexOfSelectedItem]];
    selectedOrientation = orientation[[orientationCombo indexOfSelectedItem]];
    setDefault("exportDevice",selectedDeviceType);
    setDefault("exportOrientation",selectedOrientation);
    setDefault("export2x",enableExportImage2X);
    
    dialogShutDown();
    result.option = "export";
    callback(result);
  }];
  [cancelButton setCOSJSTargetFunction:function(sender) {
    dialogShutDown();
  }];
  [bottomActionsView addSubview:exportButton];
  [bottomActionsView addSubview:cancelButton];
  [exportWindow setDefaultButtonCell:[exportButton cell]];

  var index = 0;
  var  processor = new MochaJSDelegate({
    "processArtboard:":(function(timer) {
      var skArtboard=skArtboards[index];
      var wSkDoc = sketchDoc.sketchObject;
      var fileFolder = tempFolder.path()+"/images/";
      var uuid = NSString.stringWithUUID();
      var outFile = fileFolder+ uuid+".png";

      var exportOptions= skArtboard.sketchObject.exportOptions();
      exportOptions.removeAllExportFormats();
      var exportSize = exportOptions.addExportFormat();
      var scale = 1.0;
      if(skArtboard.sketchObject.frame().width() >=skArtboard.sketchObject.frame().height())
        scale = previewWidth /skArtboard.sketchObject.frame().width();
      else
        scale = previewHeight /skArtboard.sketchObject.frame().width();
      exportSize.setScale(scale);

      var exportFormats = exportOptions.exportFormats();
      var slice = MSExportRequest.exportRequestFromExportFormat_layer_inRect_useIDForName(exportSize,skArtboard.sketchObject,skArtboard.sketchObject.absoluteRect().rect(),false);
      var hasBackgroundColor = skArtboard.sketchObject.hasBackgroundColor();
      if(hasBackgroundColor==false){
        slice.includeArtboardBackground = true;
        slice.setBackgroundColor(MSImmutableColor.whiteColor());
      }

      wSkDoc.saveExportRequest_toFile(slice, outFile);
      var preview = NSImage.alloc().initByReferencingFile(outFile);
      var artboardView = mapPreviews[index];
      [artboardView setImage: preview];

      exportOptions.removeExportFormat(exportSize);

      index += 1;
      [activityIndicator setDoubleValue:(index+1)];
      [[exportWindow contentView] addSubview:loadingView positioned:NSWindowAbove relativeTo:null];
      if (index >= artboardsCount) {
        timer.invalidate();
        [activityIndicator setHidden:true];
        [loadingView setHidden:true];
        [exportAll setEnabled:true];
        [deviceTypeCombo setEnabled:true];
        [export2x setEnabled:true];
        [exportButton setEnabled: true];
        [cancelButton setEnabled: true];
        for(var i=0;i<artboardsCount;i++){
          var artboardCheckbox = mapCheckboxes[i];
          [artboardCheckbox setEnabled:true];
        }
      }
    }),
  });


  var timer = [NSTimer scheduledTimerWithTimeInterval: 0.001 target: processor.getClassInstance() selector: "processArtboard:" userInfo: nil repeats: true]
  var runLoop = [NSRunLoop currentRunLoop];
  [runLoop addTimer:timer forMode:NSRunLoopCommonModes];
  [[app mainWindow] beginSheet:exportWindow completionHandler:nil];

}

function openSaveFileDialog(){
  removeFolderContents(tempFolder);
  var mpFilename = nsDoc.displayName().replace(".sketch", "") + ".mp";

  var panel = [NSSavePanel savePanel];
  var allowedTypes = NSArray.arrayWithObject("mp");

  panel.setAllowedFileTypes(allowedTypes);
  panel.setExtensionHidden(false);
  panel.setNameFieldStringValue(mpFilename);
  
  var result = panel.runModal();
  if (result == 0) {
    return null;
  }
  return panel.URL().path();
}

function openProgressDialog(filePath, callback){
  var width = 455;
  var height = 190;

  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var progressWindow = [[NSWindow alloc] init];
  [progressWindow setFrame:NSMakeRect(0, 0, 455, 190) display: false];
  var contentView = [progressWindow contentView];
  [progressWindow setBackgroundColor:NSColor.whiteColor()];

  var imageView = getCenterLogo(width, 106, 50);
  [contentView addSubview:imageView];

  var titleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 66, 355, 20)];
  [titleField setAlignment:NSCenterTextAlignment];
  [titleField setEditable:false];
  [titleField setBordered:false];
  [titleField setDrawsBackground:false];
  [titleField setFont:[NSFont boldSystemFontOfSize:15]];
  [titleField setStringValue:text.export_title];
  [contentView addSubview:titleField];

  var subtitleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 10, 355, 25)];
  [subtitleField setAlignment:NSCenterTextAlignment];
  [subtitleField setEditable:false];
  [subtitleField setBordered:false];
  [subtitleField setFont:[NSFont systemFontOfSize:11]];
  [subtitleField setTextColor:NSColor.colorWithGray(0.4)]
  [subtitleField setDrawsBackground:false];
  [subtitleField setStringValue:text.exporting];

  [[progressWindow contentView] addSubview:subtitleField];

  var activityIndicator = [[NSProgressIndicator alloc]initWithFrame:NSMakeRect(75,38,305,20)];
  var progressCount = artboardsToExport.length+2;
  [activityIndicator setStyle:NSProgress​Indicator​Bar​Style];
  [activityIndicator setBezeled:true];
  [activityIndicator setMinValue:0];
  [activityIndicator setMaxValue:progressCount];
  [activityIndicator setDoubleValue:0];
  [activityIndicator setIndeterminate:false];
  [[progressWindow contentView] addSubview:activityIndicator];

  var result = {option: "", extra: {}};
  var openFileButton = [[NSButton alloc] initWithFrame:NSMakeRect(178, 12, 100, 30)];
  [openFileButton setTitle:text.open_file];
  [openFileButton setBezelStyle:NSRoundedBezelStyle];
  [openFileButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [openFileButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    result.option = "openFile";
    result.extra.path = filePath;
    callback(result);
  }];
  [openFileButton setAction:"callAction:"];
  [openFileButton setHidden:true];
  [[progressWindow contentView] addSubview:openFileButton];

  var okButton = [[NSButton alloc] initWithFrame:NSMakeRect(178, 12, 100, 30)];
  [okButton setTitle:text.ok];
  [okButton setBezelStyle:NSRoundedBezelStyle];
  [okButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [okButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  }];
  [okButton setAction:"callAction:"];
  [okButton setHidden:true];
  [[progressWindow contentView] addSubview:okButton];

  var progressIndex = 0;
  var sketchData = [];
  var  processor = new MochaJSDelegate({
    "processStep:":(function(timer) {
      try{
        [activityIndicator setDoubleValue:progressIndex];
        [subtitleField setStringValue:text.exporting];
        if (progressIndex ==0) {
          unzipMpFile(tempFolder);
        }else if(progressIndex<(progressCount-1)){
          var skArtboard=artboardsToExport[progressIndex-1];
          sketchData.push(getArtboardData(skArtboard));
        }else if(progressIndex==(progressCount-1)){
           result.option = "saveFile";
           result.extra.data = sketchData;
           result.extra.path = filePath;
           callback(result);
        }

        if (progressIndex > progressCount) {
          timer.invalidate();
          [titleField setStringValue:text.export_success];
          [openFileButton setHidden:false];
          [activityIndicator setHidden:true];
          [subtitleField setHidden:true];
        }
        progressIndex++;
      }catch(e) {
        timer.invalidate();
        [titleField setStringValue:text.export_failed];
        alert(text.error,e);
        [okButton setHidden:false];
        [activityIndicator setHidden:true];
        [subtitleField setHidden:true];
      }
    }),
  });

  var timer = [NSTimer scheduledTimerWithTimeInterval: 0.05 target: processor.getClassInstance() selector: "processStep:" userInfo: nil repeats: true]
  var runLoop = [NSRunLoop currentRunLoop];
  [runLoop addTimer:timer forMode:NSRunLoopCommonModes];

  [[app mainWindow] beginSheet:progressWindow completionHandler:nil];
}

function openUpdatePluginDialog(responseData, windowProxy) {
  
  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var updateWindow = [[NSWindow alloc] init];
  var width = 455;
  var height = 240;

  [updateWindow setFrame:NSMakeRect(0, 0, width, height+20) display: false];
  var contentView = [updateWindow contentView];
  [updateWindow setBackgroundColor:NSColor.whiteColor()];

  var headerView = [[NSView alloc] initWithFrame:NSMakeRect(0, height-56, width, 56)];
  headerView.setWantsLayer(true);
  [[updateWindow contentView] addSubview:headerView];

  var fontManager = [NSFontManager sharedFontManager];
  var titleField = [[NSTextField alloc] initWithFrame:NSMakeRect(0, 0, 0, 0)];
  [titleField setEditable:false];
  [titleField setBordered:false];
  [titleField setDrawsBackground:false];
  var titleFont =[fontManager fontWithFamily:"Helvetica" traits:NSBoldFontMask weight:7 size:15];
  [titleField setFont:titleFont];
  [titleField setStringValue:text.check_new_version];
  var titleWidth = titleField.fittingSize().width;
  titleField.frame = NSMakeRect((width-titleWidth)/2, 23,titleWidth, 25);
  [headerView addSubview:titleField];

  var subtitleField = [[NSTextField alloc] initWithFrame:NSMakeRect(0, 0, 0, 0)];
  [subtitleField setEditable:false];
  [subtitleField setBordered:false];
  var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
  [subtitleField setFont:subtitleFont];
  [subtitleField setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
  [subtitleField setDrawsBackground:false];
  [subtitleField setStringValue:responseData.message];
  var subtitleWidth = subtitleField.fittingSize().width;
  subtitleField.frame = NSMakeRect((width-subtitleWidth)/2, 0,subtitleWidth, 25);
  [headerView addSubview:subtitleField];

  var b = [[NSView alloc] initWithFrame:NSMakeRect(27, 0,width-54, 1)];
  b.setWantsLayer(true);
  [b setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];
  [headerView addSubview:b];

  var logText = [[NSString alloc] initWithFormat:responseData.value.updateLog];
  var logInfoField = [[NSTextField alloc] initWithFrame:CGRectMake(0,0,0,0)];
  var logFont = [NSFont systemFontOfSize:15];
  [logInfoField setStringValue:logText];
  [logInfoField setEditable:false];
  [logInfoField setBordered:false];
  [logInfoField setFont:logFont];
  [logInfoField setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
  [logInfoField setDrawsBackground:false];

  //自动计算文本的高度
  var attribute = [NSDictionary dictionaryWithObjectsAndKeys: logFont,NSFontAttributeName,nil];
  var option = NSStringDrawingUsesLineFragmentOrigin|NSStringDrawingUsesFontLeading;
  var size = CGSizeMake(width-60, 0);
  var txRect = [logText boundingRectWithSize:size options:option attributes:attribute context:nil];
  logInfoField.frame=CGRectMake(30, 60, width-60, txRect.size.height);


  var scrollLogInfo = [[NSScrollView alloc] initWithFrame:NSMakeRect(20, height-56-140, width, 138)];
  [scrollLogInfo setHasVerticalScroller:true];
  [scrollLogInfo setHasHorizontalScroller:true];
  [scrollLogInfo setBorderType:NSNoBorder];
  [scrollLogInfo setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
  [scrollLogInfo setDocumentView:logInfoField];
  [scrollLogInfo setDrawsBackground:false];
  [scrollLogInfo setAutohidesScrollers:true];
  [[scrollLogInfo verticalScroller] setScrollerStyle:NSScrollerStyleLegacy];
  [[updateWindow contentView] addSubview:scrollLogInfo];

  var result = {option: "", extra: {}};
  var updateButton = [[NSButton alloc] initWithFrame:NSMakeRect(123, 5, 92, 30)];
  [updateButton setTitle:text.update];
  [updateButton setBezelStyle:NSRoundedBezelStyle];
  [updateButton setCOSJSTargetFunction:function(sender) {
    [updateWindow orderOut:nil];
    [[app mainWindow] endSheet: updateWindow];
    [updateButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    result.option = "update";
    result.extra.downloadURL = responseData.value.downloadURL;
    windowProxy.callback(result);
  }];
  [updateButton setAction:"callAction:"];
  
  ignoreButton = [[NSButton alloc] initWithFrame:NSMakeRect(240, 5, 92, 30)];
  [ignoreButton setTitle:text.ignore];
  [ignoreButton setBezelStyle:NSRoundedBezelStyle];
  [ignoreButton setAction:"callAction:"];

  [ignoreButton setCOSJSTargetFunction:function(sender) {
    [updateWindow orderOut:nil];
    [[app mainWindow] endSheet: updateWindow];
    [updateButton setCOSJSTargetFunction:undefined];
    [ignoreButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    result.option = "ignore";
    windowProxy.callback(result);
  }];

  var bottomActionsView = [[NSView alloc] initWithFrame:NSMakeRect(0, 0,width, 42)];
  [[updateWindow contentView] addSubview:bottomActionsView];
  var border = [[NSView alloc] initWithFrame:NSMakeRect(27, 41,width-54, 1)];
  [bottomActionsView addSubview:border];
  border.setWantsLayer(true);
  [border setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];

  [bottomActionsView addSubview:updateButton];
  [bottomActionsView addSubview:ignoreButton];
  [updateWindow setDefaultButtonCell:[updateButton cell]];
  [[app mainWindow] beginSheet:updateWindow completionHandler:nil];
  
}

function openLoginDialog(windowProxy) {
  var width = 400;
  var height = 350;

  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var loginWindow = [[NSWindow alloc] init];
  [loginWindow setFrame:NSMakeRect(0, 0, 400, 350) display: false];
  var contentView = [loginWindow contentView];
  [loginWindow setBackgroundColor:NSColor.whiteColor()];

  var imageView = getCenterLogo(width, height-108, 60);
  [contentView addSubview:imageView];

  var name = _getValueForKey("email") || "";
  var password = _getValueForKey("password") || "";
  var isRemmenber = _getValueForKey("remmenberPassword") == "true";
  
  log("test1");
  
  var usernameField = getInputText(NSMakeRect(60, 181, 280, 24), name, text.username_placeholder);
  [contentView addSubview:usernameField];

  var passwordField = getInputText(NSMakeRect(60, 140, 280, 24), password, text.password_placeholder, true);
  [[loginWindow contentView] addSubview:passwordField];

  var remmenberPassword =[[NSButton alloc] initWithFrame: NSMakeRect(60, 104, 300, 25)];
  [remmenberPassword setButtonType:NSSwitchButton];
  [remmenberPassword setTitle: text.remmenber_password];
  [remmenberPassword setFont:[NSFont systemFontOfSize:13]];
  if(isRemmenber) {
    [remmenberPassword setState:NSOnState];
  }
  [[loginWindow contentView] addSubview:remmenberPassword];


  var forgetButton = getLinkButton(text.forget_password, text.forget_password, NSMakeRect(400-160, 102, 150, 30), function() {
      windowProxy.shutDownCallback("forget");
    });

  [[loginWindow contentView] addSubview:forgetButton];
  
  var result = {option: "", extra: {}};
  var dialogShutDown = function() {
    [loginWindow orderOut:nil];
    [[app mainWindow] endSheet: loginWindow];
    [loginButton setCOSJSTargetFunction:undefined];
    [cancelButton setCOSJSTargetFunction:undefined];
    [registButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  };
  var loginButton = [[NSButton alloc] initWithFrame:NSMakeRect(53, 70, 140, 24)];
  [loginButton setTitle:text.login];
  [loginButton setBezelStyle:NSRoundedBezelStyle];
  [loginButton setCOSJSTargetFunction:function(sender) {
    var userInfo = {};
    userInfo.email = [usernameField stringValue];
    userInfo.password = [passwordField stringValue];
    _setValueForKey(userInfo.email, "email");
    if([remmenberPassword state] == NSOnState) {
      _setValueForKey("true", "remmenberPassword");
      _setValueForKey(userInfo.password, "password");
    } else {
      _setValueForKey("false", "remmenberPassword");
      _setValueForKey("", "password");
    }
    windowProxy.handleLoginRequest(userInfo, function(res) {
      if(res.errMsg) {
        [nsDoc showMessage: res.errMsg];
        log(res.errMsg);
       return;
      }
      dialogShutDown();
      [nsDoc showMessage: text.login_success];
      windowProxy.shutDownCallback("loginSuccess");
    });
  }];
  [loginButton setAction:"callAction:"];
  [loginButton setHidden:false];
  [[loginWindow contentView] addSubview:loginButton];

  var cancelButton = [[NSButton alloc] initWithFrame:NSMakeRect(207, 70, 140, 24)];
  [cancelButton setTitle:text.cancel];
  [cancelButton setBezelStyle:NSRoundedBezelStyle];
  [cancelButton setCOSJSTargetFunction:function(sender) {
    dialogShutDown();
  }];
  [cancelButton setAction:"callAction:"];
  [cancelButton setHidden:false];
  [[loginWindow contentView] addSubview:cancelButton];

  var registLabel = [[NSTextField alloc] initWithFrame:NSMakeRect(60, 28, 100, 30)];
  [registLabel setEditable:false];
  [registLabel setBordered:false];
  [registLabel setFont:[NSFont systemFontOfSize:13]];
  [registLabel setTextColor:NSColor.colorWithGray(0.4)]
  [registLabel setDrawsBackground:false];
  [registLabel setStringValue:text.first_use];
  var labelWidth = registLabel.fittingSize().width;
  var begin = (400-120-labelWidth)/2;
  registLabel.frame = NSMakeRect(begin, 28,labelWidth, 30);
  [[loginWindow contentView] addSubview:registLabel];

  var registButton = [[NSButton alloc] initWithFrame:NSMakeRect(labelWidth+begin, 32, 120, 30)];
  [registButton setTitle:text.regist];
  [registButton setShowsBorderOnlyWhileMouseInside: true]
  [registButton setBezelStyle:NSRoundedBezelStyle];
  [registButton setCOSJSTargetFunction:function(sender) {
    [loginWindow orderOut:nil];
    [[app mainWindow] endSheet: loginWindow];
    [registButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    windowProxy.shutDownCallback("regist");
  }];
  [registButton setAction:"callAction:"];
  [registButton setHidden:false];
  [[loginWindow contentView] addSubview:registButton];

  [loginWindow setDefaultButtonCell:[loginButton cell]];
  [[app mainWindow] beginSheet:loginWindow completionHandler:nil];
}

function openRegistDialog(windowProxy) {
  var width = 400;
  var height = 340;

  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var registWindow = [[NSWindow alloc] init];
  [registWindow setFrame:NSMakeRect(0, 0, width, height) display: false];
  var contentView = [registWindow contentView];
  [registWindow setBackgroundColor:NSColor.whiteColor()];

  var imageView = getCenterLogo(width, height-108, 60);
  [contentView addSubview:imageView];

  var usernameField = getInputText(NSMakeRect(60, 181, 280, 24), "", text.username_placeholder);
  [contentView addSubview:usernameField];

  var passwordField = getInputText(NSMakeRect(60, 140, 280, 24), "", text.password_placeholder, true);
  [contentView addSubview:passwordField];

  var confirePasswordField = getInputText(NSMakeRect(60, 99, 280, 24), "", text.fonfire_password, true);
  [contentView addSubview:confirePasswordField];

  var result = {option: "", extra: {}};

  var registButton = [[NSButton alloc] initWithFrame:NSMakeRect(53, 52, 294, 30)];
  [registButton setTitle:text.regist_button];
  [registButton setBezelStyle:NSRoundedBezelStyle];
  [registButton setCOSJSTargetFunction:function(sender) {
    var userInfo = {};
    userInfo.email = [usernameField stringValue];
    userInfo.password = [passwordField stringValue];
    var confirePassword = [confirePasswordField stringValue];
    if(confirePassword != userInfo.password) {
      [nsDoc showMessage: password_not_same];
      return;
    }
    windowProxy.handleRegistRequest(userInfo, function(res) {
      if(res.errMsg) {
        [nsDoc showMessage: res.errMsg];
        log(res.errMsg);
        return;
      }
      [registWindow orderOut:nil];
      [[app mainWindow] endSheet: registWindow];
      [registButton setCOSJSTargetFunction:undefined];
      COScript.currentCOScript().setShouldKeepAround_(false);
      [nsDoc showMessage: text.regist_success];
      windowProxy.shutDownCallback("registSuccess");
    });
  }];
  [registButton setAction:"callAction:"];
  [registButton setHidden:false];
  [[registWindow contentView] addSubview:registButton];

  var registLabel = [[NSTextField alloc] initWithFrame:NSMakeRect(60, 10, 100, 30)];
  [registLabel setEditable:false];
  [registLabel setBordered:false];
  [registLabel setFont:[NSFont systemFontOfSize:13]];
  [registLabel setTextColor:NSColor.colorWithGray(0.4)]
  [registLabel setDrawsBackground:false];
  [registLabel setStringValue:text.have_username];
  var labelWidth = registLabel.fittingSize().width;
  var begin = (400-100-labelWidth)/2;
  registLabel.frame = NSMakeRect(begin, 10,labelWidth, 30);
  [[registWindow contentView] addSubview:registLabel];

  var loginButton = [[NSButton alloc] initWithFrame:NSMakeRect(begin+labelWidth, 14, 100, 30)];
  [loginButton setTitle:text.start_login];
  [loginButton setShowsBorderOnlyWhileMouseInside: true]
  [loginButton setBezelStyle:NSRoundedBezelStyle];
  [loginButton setCOSJSTargetFunction:function(sender) {
    [registWindow orderOut:nil];
    [[app mainWindow] endSheet: registWindow];
    [loginButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    windowProxy.shutDownCallback("login");
  }];
  [loginButton setAction:"callAction:"];
  [loginButton setHidden:false];
  [[registWindow contentView] addSubview:loginButton];


  [registWindow setDefaultButtonCell:[registButton cell]];
  [[app mainWindow] beginSheet:registWindow completionHandler:nil];
}

function openUploadDialog(callback){
  var sid = _getValueForKey("sid");
  var isPro = _getValueForKey("version") == "pro";
  var app = [NSApplication sharedApplication];
  var exportWindow = [[NSWindow alloc] init];
  var horizontalMargin = 35;
  var width = 770;
  var height = 550;
  var offset = 0;
  if(!sid) {
    height += 20;
    offset = 20;
  } else if(!isPro) {
    height += 35;
    offset = 35;
  }

  COScript.currentCOScript().setShouldKeepAround_(true);
  [exportWindow setFrame:NSMakeRect(0, 0, width, height+20) display: false];
  var contentView = [exportWindow contentView];
  contentView.setWantsLayer(true);
  [contentView setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(235/255, 235/255, 235/255, 0.7)];

  var headerView = [[NSView alloc] initWithFrame:NSMakeRect(0, height-146, width, 146)];
  headerView.setWantsLayer(true);
  [headerView setBackgroundColor:NSColor.whiteColor()];
  [[exportWindow contentView] addSubview:headerView];

  var imageView = getCenterLogo(width, 76, 50);
  [headerView addSubview:imageView];

  var titleField = getCenterTitle(width, 38, 25, text.upload_title);
  [headerView addSubview:titleField];

  var subtitleField = getCenterSubtitle(width, 12, 25, text.upload_subtitle);
  [headerView addSubview:subtitleField];

  var fontManager = [NSFontManager sharedFontManager];

  var skArtboards = getAllArtboards(sketchDoc);
  var artboardsCount = skArtboards.length;
  var previewWidth = 76;
  var previewHeight = 76;
  
  var container = artboardScrollFragment.createContainer();
  var scrollArtboards = getScrollView(NSMakeRect(20, height-146-250-35, 464, 250), container);
  [[exportWindow contentView] addSubview:scrollArtboards];
  var mapCheckboxes = artboardScrollFragment.getMapCheckBoxes();
  var mapPreviews = artboardScrollFragment.getMapPreviews();

  var exportAll =[[NSButton alloc] initWithFrame: NSMakeRect(25, height-470, 200, 20)];
  [exportAll setButtonType:NSSwitchButton];
  [exportAll setTitle:text.publish_all];
  [exportAll setFont:[NSFont systemFontOfSize:13]];
  var allselected = selectedArtboards.length==0 || selectedArtboards.length==artboardsCount;
  [exportAll setState:(allselected ? NSOnState : NSOffState)];
  [exportAll setEnabled:false];
  [[exportWindow contentView] addSubview:exportAll];
  [exportAll setCOSJSTargetFunction:function(sender) {
    for(var i=0;i<artboardsCount;i++){
      var artboardCheckbox = mapCheckboxes[i];
      [artboardCheckbox setState:[exportAll state]];
    }
  }];


  var borderVertical = [[NSView alloc] initWithFrame:NSMakeRect(499, height-146-316,1, 282)];
  [[exportWindow contentView] addSubview:borderVertical];
  borderVertical.setWantsLayer(true);
  [borderVertical setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];

  var projectComboView = projectComboFragment.getProjectCombo(NSMakeRect(514, height-280, 300, 100));
  var deviceTypeCombo = projectComboFragment.getDeviceTypeCombo();
  var orientationCombo = projectComboFragment.getOrientationCombo();
  [[exportWindow contentView] addSubview:projectComboView];

  [deviceTypeCombo setCOSJSTargetFunction:function(sender) {
    var projectIndex = [deviceTypeCombo indexOfSelectedItem];
    if([0,1].indexOf(projectIndex) === -1)
      [orientationCombo setEnabled: false];
    else
      [orientationCombo setEnabled: true];
  }];

  var export2x =[[NSButton alloc] initWithFrame: NSMakeRect(514, height-334, 300, 25)];
  [export2x setButtonType:NSSwitchButton];
  [export2x setTitle: text.publish_2x];
  [export2x setFont:[NSFont systemFontOfSize:13]];
  if(getDefault("export2x",0)==1)
    [export2x setState:NSOnState];
  [export2x setEnabled:false];
  [[exportWindow contentView] addSubview:export2x];

  var exportButton = [[NSButton alloc] initWithFrame:NSMakeRect(397, 15, 92, 30)],
  cancelButton = [[NSButton alloc] initWithFrame:NSMakeRect(280, 15, 92, 30)];
  [exportButton setEnabled: false];
  [exportButton setEnabled: false];
  [exportButton setTitle:text.publish];
  [exportButton setBezelStyle:NSRoundedBezelStyle];
  [exportButton setAction:"callAction:"];
  [cancelButton setTitle:text.cancel];
  [cancelButton setBezelStyle:NSRoundedBezelStyle];
  [cancelButton setAction:"callAction:"];

  var bottomActionsView = [[NSView alloc] initWithFrame:NSMakeRect(0, 0,width, 68+offset)];
  [[exportWindow contentView] addSubview:bottomActionsView];
  var border = [[NSView alloc] initWithFrame:NSMakeRect(27, 67+offset,width-54, 1)];
  [bottomActionsView addSubview:border];
  border.setWantsLayer(true);
  [border setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(210/255, 210/255, 210/255, 1.0)];


  var loadingView = [[NSView alloc] initWithFrame:NSMakeRect(0, 0, width, height-146)];
  loadingView.setWantsLayer(true);
  [loadingView setBackgroundColor:NSColor.colorWithRed_green_blue_alpha(255/255, 255/255, 255/255, 0.7)];
  [[exportWindow contentView] addSubview:loadingView positioned:NSWindowAbove relativeTo:null];

  var activityIndicator = [[NSProgressIndicator alloc]initWithFrame:NSMakeRect(190,245,width-320,NSProgressIndicatorPreferredLargeThickness)];
  [activityIndicator setStyle:NSProgress​Indicator​Bar​Style];
  [activityIndicator setBezeled:true];
  [activityIndicator setMinValue:0];
  [activityIndicator setMaxValue:artboardsCount];
  [activityIndicator setDoubleValue:0];
  [activityIndicator setIndeterminate:false];
  [loadingView addSubview:activityIndicator];

  var result = {option: "", extra: {}};
  [exportButton setCOSJSTargetFunction:function(sender) {
    artboardsToExport = [];
    for(var i=0;i<artboardsCount;i++){
      var artboardCheckbox = mapCheckboxes[i];
      if([artboardCheckbox state]==NSOnState){
        artboardsToExport.push(skArtboards[i]);
      }
    }
    if(artboardsToExport.length==0){
        alert(text.uncheck_artboard_title, text.uncheck_artboard_content);
      return;
    }
    
    enableExportImage2X = [export2x state] == NSOnState;
    selectedDeviceType = deviceTypes[[deviceTypeCombo indexOfSelectedItem]];
    selectedOrientation = orientation[[orientationCombo indexOfSelectedItem]];
    setDefault("exportDevice",selectedDeviceType);
    setDefault("exportOrientation",selectedOrientation);
    setDefault("export2x",enableExportImage2X);
    [exportWindow orderOut:nil];
    [[app mainWindow] endSheet: exportWindow];
    [exportButton setCOSJSTargetFunction:undefined];
    [cancelButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);

    result.option = "upload";
    callback(result);
  }];

  [cancelButton setCOSJSTargetFunction:function(sender) {
    [exportWindow orderOut:nil];
    [[app mainWindow] endSheet: exportWindow];
    [exportButton setCOSJSTargetFunction:undefined];
    [cancelButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  }];

  var canPublish = true;
  if(!sid) {
    var versionMsg = [[NSTextField alloc] initWithFrame:NSMakeRect(20, 48, width-40, 25)];
    [versionMsg setEditable:false];
    [versionMsg setBordered:false];
    var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
    [versionMsg setFont:subtitleFont];
    [versionMsg setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
    [versionMsg setDrawsBackground:false];
    [versionMsg setStringValue: text.no_username_message];
    var w = versionMsg.fittingSize().width;
    
    var btn = [[NSButton alloc] initWithFrame:NSMakeRect(w+20, 50, 150, 25)];
    [btn setTitle:text.no_username_handle];
    [btn setShowsBorderOnlyWhileMouseInside: true]
    [btn setBezelStyle:NSRoundedBezelStyle];
    [btn setCOSJSTargetFunction:function(sender) {
      [exportWindow orderOut:nil];
      [[app mainWindow] endSheet: exportWindow];
      [exportButton setCOSJSTargetFunction:undefined];
      [cancelButton setCOSJSTargetFunction:undefined];
      COScript.currentCOScript().setShouldKeepAround_(false);
      result.option = "login";
      callback(result);
    }];
    [btn setAction:"callAction:"];
    [btn setHidden:false];

    [bottomActionsView addSubview: versionMsg];
    [bottomActionsView addSubview:btn];
    canPublish = false;
  } else if(!isPro) {
    var versionMsg = [[NSTextField alloc] initWithFrame:NSMakeRect(20, 68, width-40, 25)];
    [versionMsg setEditable:false];
    [versionMsg setBordered:false];
    var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
    [versionMsg setFont:subtitleFont];
    [versionMsg setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
    [versionMsg setDrawsBackground:false];
    [versionMsg setStringValue: text.not_pro_message];
    var w = versionMsg.fittingSize().width;
    
    var btn = getLinkButton(text.update_to_pro, text.update_to_pro, NSMakeRect(w+18, 71, 120, 25), function() {
      result.option = "buy";
      callback(result);
    });

    var versionMsg2 = [[NSTextField alloc] initWithFrame:NSMakeRect(w+140, 68, width-40, 25)];
    [versionMsg2 setEditable:false];
    [versionMsg2 setBordered:false];
    var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
    [versionMsg2 setFont:subtitleFont];
    [versionMsg2 setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
    [versionMsg2 setDrawsBackground:false];
    [versionMsg2 setStringValue: text.relogin_message];
    
    var versionMsg3 = [[NSTextField alloc] initWithFrame:NSMakeRect(20, 45, width-40, 25)];
    [versionMsg3 setEditable:false];
    [versionMsg3 setBordered:false];
    var subtitleFont =[fontManager fontWithFamily:"Helvetica" traits:0 weight:0 size:13];
    [versionMsg3 setFont:subtitleFont];
    [versionMsg3 setTextColor:NSColor.colorWithRed_green_blue_alpha(69/255, 69/255, 69/255, 1.0)];
    [versionMsg3 setDrawsBackground:false];
    [versionMsg3 setStringValue: " after the upgrade is complete. "]; 

    [bottomActionsView addSubview: versionMsg];
    [bottomActionsView addSubview: versionMsg2];
    [bottomActionsView addSubview: versionMsg3];
    [bottomActionsView addSubview:btn];
    canPublish = false;
  } else {
    [exportWindow setDefaultButtonCell:[exportButton cell]];
  }
  [bottomActionsView addSubview:exportButton];
  [bottomActionsView addSubview:cancelButton];

  var index = 0;

  var  processor = new MochaJSDelegate({
    "processArtboard:":(function(timer) {
      var skArtboard=skArtboards[index];
      var wSkDoc = sketchDoc.sketchObject;
      var fileFolder = tempFolder.path()+"/images/";
      var uuid = NSString.stringWithUUID();
      var outFile = fileFolder+ uuid+".png";

      var exportOptions= skArtboard.sketchObject.exportOptions();
      exportOptions.removeAllExportFormats();
      var exportSize = exportOptions.addExportFormat();
      var scale = 1.0;
      if(skArtboard.sketchObject.frame().width() >=skArtboard.sketchObject.frame().height())
        scale = previewWidth /skArtboard.sketchObject.frame().width();
      else
        scale = previewHeight /skArtboard.sketchObject.frame().width();
      exportSize.setScale(scale);

      var exportFormats = exportOptions.exportFormats();
      var slice = MSExportRequest.exportRequestFromExportFormat_layer_inRect_useIDForName(exportSize,skArtboard.sketchObject,skArtboard.sketchObject.absoluteRect().rect(),false);
      var hasBackgroundColor = skArtboard.sketchObject.hasBackgroundColor();
      if(hasBackgroundColor==false){
        slice.includeArtboardBackground = true;
        slice.setBackgroundColor(MSImmutableColor.whiteColor());
      }

      wSkDoc.saveExportRequest_toFile(slice, outFile);
      var preview = NSImage.alloc().initByReferencingFile(outFile);
      var artboardView = mapPreviews[index];
      [artboardView setImage: preview];

      exportOptions.removeExportFormat(exportSize);

      index += 1;
      [activityIndicator setDoubleValue:(index+1)];
      [[exportWindow contentView] addSubview:loadingView positioned:NSWindowAbove relativeTo:null];
      if (index >= artboardsCount) {
        timer.invalidate();
        [activityIndicator setHidden:true];
        [loadingView setHidden:true];
        [exportAll setEnabled:true];
        [deviceTypeCombo setEnabled:true];
        [export2x setEnabled:true];
        [exportButton setEnabled: canPublish];
        [cancelButton setEnabled: true];
        for(var i=0;i<artboardsCount;i++){
          var artboardCheckbox = mapCheckboxes[i];
          [artboardCheckbox setEnabled:true];
        }
      }
    }),
  });


  var timer = [NSTimer scheduledTimerWithTimeInterval: 0.001 target: processor.getClassInstance() selector: "processArtboard:" userInfo: nil repeats: true]
  var runLoop = [NSRunLoop currentRunLoop];
  [runLoop addTimer:timer forMode:NSRunLoopCommonModes];
  [[app mainWindow] beginSheet:exportWindow completionHandler:nil];

}

function openUploadProgressDialog(filePath, windowProxy){
  var width = 455;
  var height = 190;

  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var progressWindow = [[NSWindow alloc] init];
  [progressWindow setFrame:NSMakeRect(0, 0, 455, 190) display: false];
  var contentView = [progressWindow contentView];
  [progressWindow setBackgroundColor:NSColor.whiteColor()];

  var imageView = getCenterLogo(width, 106, 50);
  [contentView addSubview:imageView];

  var titleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 66, 355, 20)];
  [titleField setAlignment:NSCenterTextAlignment];
  [titleField setEditable:false];
  [titleField setBordered:false];
  [titleField setDrawsBackground:false];
  [titleField setFont:[NSFont boldSystemFontOfSize:15]];
  [titleField setStringValue:text.publish_to_cloud];
  [contentView addSubview:titleField];

  var subtitleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 10, 355, 25)];
  [subtitleField setAlignment:NSCenterTextAlignment];
  [subtitleField setEditable:false];
  [subtitleField setBordered:false];
  [subtitleField setFont:[NSFont systemFontOfSize:11]];
  [subtitleField setTextColor:NSColor.colorWithGray(0.4)]
  [subtitleField setDrawsBackground:false];
  [subtitleField setStringValue:text.exporting];

  [[progressWindow contentView] addSubview:subtitleField];

  var activityIndicator = [[NSProgressIndicator alloc]initWithFrame:NSMakeRect(75,38,305,20)];
  var progressCount = artboardsToExport.length+2;
  [activityIndicator setStyle:NSProgress​Indicator​Bar​Style];
  [activityIndicator setBezeled:true];
  [activityIndicator setMinValue:0];
  [activityIndicator setMaxValue:progressCount];
  [activityIndicator setDoubleValue:0];
  [activityIndicator setIndeterminate:false];
  [[progressWindow contentView] addSubview:activityIndicator];

  var result = {option: "", extra: {}};
  var viewProjectButton = [[NSButton alloc] initWithFrame:NSMakeRect(152, 12, 150, 30)];
  [viewProjectButton setTitle:text.view_project];
  [viewProjectButton setBezelStyle:NSRoundedBezelStyle];
  [viewProjectButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [viewProjectButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    windowProxy.callback("viewProject");
  }];
  [viewProjectButton setAction:"callAction:"];
  [viewProjectButton setHidden:true];
  [[progressWindow contentView] addSubview:viewProjectButton];

  var okButton = [[NSButton alloc] initWithFrame:NSMakeRect(178, 12, 100, 30)];
  [okButton setTitle:text.ok];
  [okButton setBezelStyle:NSRoundedBezelStyle];
  [okButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [okButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  }];
  [okButton setAction:"callAction:"];
  [okButton setHidden:true];
  [[progressWindow contentView] addSubview:okButton];

  var loginButton = [[NSButton alloc] initWithFrame:NSMakeRect(178, 12, 100, 30)];
  [loginButton setTitle:text.relogin];
  [loginButton setBezelStyle:NSRoundedBezelStyle];
  [loginButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [loginButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    windowProxy.callback("login");
  }];
  [loginButton setAction:"callAction:"];
  [loginButton setHidden:true];
  [[progressWindow contentView] addSubview:loginButton];

  var progressIndex = 0;
  var sketchData = [];
  var  processor = new MochaJSDelegate({
    "processStep:":(function(timer) {
      try{
        [activityIndicator setDoubleValue:progressIndex];
        [subtitleField setStringValue:text.exporting];
        if (progressIndex ==0) {
          unzipMpFile(tempFolder);
        }else if(progressIndex<(progressCount-1)){
          var skArtboard=artboardsToExport[progressIndex-1];
          sketchData.push(getArtboardData(skArtboard));
        }else if(progressIndex==(progressCount-1)){
           windowProxy.windowContext.data = sketchData;
           windowProxy.windowContext.path = filePath;
           [subtitleField setStringValue:text.publishing];
           var res = windowProxy.callback("upload");
           if(res.code == 401) {
              [titleField setStringValue:res.message];
              [activityIndicator setHidden:true];
              [loginButton setHidden: false];
              [subtitleField setHidden:true];
           } else if(res.code == 0) {
              [titleField setStringValue:text.publish_success];
              [activityIndicator setHidden:true];
              [viewProjectButton setHidden:false];
              [subtitleField setHidden:true];
           } else {
              [titleField setStringValue:res.message];
              [activityIndicator setHidden:true];
              [okButton setHidden:false];
              [subtitleField setHidden:true];
           }
        }

        if (progressIndex > progressCount) {
          timer.invalidate();
          //[subtitleField setStringValue:windowProxy.windowContext.url];
        }
        progressIndex++;
      }catch(e) {
        timer.invalidate();
        [titleField setStringValue:text.publish_failed];
        alert(text.error,e);
        [okButton setHidden:false];
        [activityIndicator setHidden:true];
        [subtitleField setHidden:true];
      }
    }),
  });

  var timer = [NSTimer scheduledTimerWithTimeInterval: 0.05 target: processor.getClassInstance() selector: "processStep:" userInfo: nil repeats: true]
  var runLoop = [NSRunLoop currentRunLoop];
  [runLoop addTimer:timer forMode:NSRunLoopCommonModes];

  [[app mainWindow] beginSheet:progressWindow completionHandler:nil];
}

function openPublishDialog(windowProxy) {
  var width = 455;
  var height = 190;

  COScript.currentCOScript().setShouldKeepAround_(true);
  var app = [NSApplication sharedApplication];
  var progressWindow = [[NSWindow alloc] init];
  [progressWindow setFrame:NSMakeRect(0, 0, 455, 190) display: false];
  var contentView = [progressWindow contentView];
  [progressWindow setBackgroundColor:NSColor.whiteColor()];

  var imageView = getCenterLogo(width, 106, 50);
  [contentView addSubview:imageView];

  var titleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 66, 355, 20)];
  [titleField setAlignment:NSCenterTextAlignment];
  [titleField setEditable:false];
  [titleField setBordered:false];
  [titleField setDrawsBackground:false];
  [titleField setFont:[NSFont boldSystemFontOfSize:15]];
  [titleField setStringValue:text.publish_to_cloud];
  [contentView addSubview:titleField];

  var subtitleField = [[NSTextField alloc] initWithFrame:NSMakeRect(50, 10, 355, 25)];
  [subtitleField setAlignment:NSCenterTextAlignment];
  [subtitleField setEditable:false];
  [subtitleField setBordered:false];
  [subtitleField setFont:[NSFont systemFontOfSize:11]];
  [subtitleField setTextColor:NSColor.colorWithGray(0.4)]
  [subtitleField setDrawsBackground:false];
  [subtitleField setStringValue:text.publishing];

  [[progressWindow contentView] addSubview:subtitleField];

  var result = {option: "", extra: {}};
  var viewProjectButton = [[NSButton alloc] initWithFrame:NSMakeRect(152, 12, 150, 30)];
  [viewProjectButton setTitle:text.view_project];
  [viewProjectButton setBezelStyle:NSRoundedBezelStyle];
  [viewProjectButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [viewProjectButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
    windowProxy.callback("viewProject");
  }];
  [viewProjectButton setAction:"callAction:"];
  [viewProjectButton setHidden:true];
  [[progressWindow contentView] addSubview:viewProjectButton];

  var okButton = [[NSButton alloc] initWithFrame:NSMakeRect(178, 12, 100, 30)];
  [okButton setTitle:text.ok];
  [okButton setBezelStyle:NSRoundedBezelStyle];
  [okButton setCOSJSTargetFunction:function(sender) {
    [progressWindow orderOut:nil];
    [[app mainWindow] endSheet: progressWindow];
    [okButton setCOSJSTargetFunction:undefined];
    COScript.currentCOScript().setShouldKeepAround_(false);
  }];
  [okButton setAction:"callAction:"];
  [okButton setHidden:true];
  [[progressWindow contentView] addSubview:okButton];
  [[app mainWindow] beginSheet:progressWindow completionHandler:nil];

  var res = windowProxy.callback("upload");
  if(res.errMsg) {
    [titleField setStringValue:text.publish_failed];
    [okButton setHidden:false];
    [subtitleField setHidden:true];
  } else {
    [titleField setStringValue:text.publish_success];
    [viewProjectButton setHidden:false];
    [subtitleField setHidden:true];
  }
}
