@import 'fileUtils.js'
@import 'utils.js'

var mapID = {};
projectProperties = {};

function createMockplusXMLData(folderURL,sketchData){
  mapID = {};
  var box = figurePageSize(sketchData);
  checkPageSize(box);
  initProjectProperties(box);
  
  writeProjectProperties(folderURL);
  for (var i = 0; i < sketchData.length; i++) {
    var artboardData = sketchData[i];
    createPageFromArtboard(folderURL,i,artboardData);
  }
}

function initProjectProperties(box) {
  projectProperties.projectName = getDefault("projectName");
  projectProperties.width = box.width;
  projectProperties.height = box.height;
  projectProperties.platform = "Mt" + getDefault("exportDevice", "Web");
  if(isOrientation(selectedDeviceType)) {
    projectProperties.orientation = "Mo" + getDefault("exportOrientation", "Portrait");
    projectProperties.mobileType = box.mobileType;
    if(!isPortrait()) {
      projectProperties.width = Math.max(box.width,box.height);
      projectProperties.height = Math.min(box.width,box.height);
    }
  }
}

function createPageFromArtboard(folderURL,index, artboardData){
  var pageID =  createNewUUID();

  var screenXMLPath = folderURL.path().stringByAppendingPathComponent("MBX").stringByAppendingPathComponent(pageID+".mbx");

  var xmlDoc = createPageXMLDoc(pageID,index, artboardData);
  writeXMLDoc(xmlDoc,screenXMLPath);

  writePageToMain(folderURL,pageID,artboardData);
}

function createPageXMLDoc(id,index, artboardData){

  var root = NSXMLNode.elementWithName("mockup");
  var xmlDoc = NSXMLDocument.alloc().initWithRootElement(root);
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("version","4.7"));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("minAllowVersion","4.7"));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("ID",id));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("platform",projectProperties.platform));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("defaultW",projectProperties.width+""));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("defaultH",projectProperties.height+""));
  if(isOrientation(selectedDeviceType)) {  
    root.addAttribute(NSXMLNode.attributeWithName_stringValue("screenOrientation",projectProperties.orientation));
    root.addAttribute(NSXMLNode.attributeWithName_stringValue("clipArea","(0,0,0,0)"));
  }
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("mobileScreenScale","1.77777779102325"));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("measureW",projectProperties.width+""));
  root.addAttribute(NSXMLNode.attributeWithName_stringValue("measureH",projectProperties.height+""));

  var controls = NSXMLNode.elementWithName("controls");
  root.addChild(controls);
  controls.addAttribute(NSXMLNode.attributeWithName_stringValue("controlCount","1"));

  controls.addChild(createControlXMLDoc(artboardData));

  var param = NSXMLNode.elementWithName("param");
  root.addChild(param);
  
  param.addChild(NSXMLNode.elementWithName_stringValue("backgroundColor", "4294967295"));
  param.addChild(NSXMLNode.elementWithName_stringValue("fontSize", "0"));
  param.addChild(NSXMLNode.elementWithName("fontFamily"));
  param.addChild(NSXMLNode.elementWithName("guideLines"));

  return xmlDoc;
}

function createControlXMLDoc(artboardData) {

  var imageData = artboardData.items[0];

  var control = NSXMLNode.elementWithName("control");
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("controlID",createNewUUID()));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("controlTypeID","Image"));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("isShow",imageData.visible ? "True" : "False"));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("caption",imageData.name+""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("x",imageData.relativePosition.x+""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("y",imageData.relativePosition.y+""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("w",imageData.size.width+""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("h",imageData.size.height+""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("measuredW",""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("measuredH",""));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("zOrder","0"));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("locked","False"));
  control.addAttribute(NSXMLNode.attributeWithName_stringValue("sizeLocked","False"));

  var controlProperties = NSXMLNode.elementWithName("controlProperties");
  control.addChild(controlProperties);

  var basic = NSXMLNode.elementWithName("basic");
  controlProperties.addChild(basic);
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("color","4294967295"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("borderVisible","False"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("borderColor","4284900966"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("isAutoSize","False"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("status","MsEnabled"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("opacity","1"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("isCommentCom","False"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("canMove","True"));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("alias",""));
  basic.addAttribute(NSXMLNode.attributeWithName_stringValue("rotation","0"));

  controlProperties.addChild(NSXMLNode.elementWithName("states"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("verticalScrollbarShow", "False"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("verticalScrollbarProcess", "0"))
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("horizontalScrollbarShow", "False"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("horizontalScrollbarProcess", "0"));

  var links = NSXMLNode.elementWithName("links");
  controlProperties.addChild(links);
  links.addAttribute(NSXMLNode.attributeWithName_stringValue("eventType","AtClick"));
  links.addAttribute(NSXMLNode.attributeWithName_stringValue("animate","AmNone"));
  links.addAttribute(NSXMLNode.attributeWithName_stringValue("animateAutoDelay","0"));
  links.addAttribute(NSXMLNode.attributeWithName_stringValue("animateDuration","500"));

  var link = NSXMLNode.elementWithName("link");
  links.addChild(link);
  link.addAttribute(NSXMLNode.attributeWithName_stringValue("targetPageID",""));
  link.addAttribute(NSXMLNode.attributeWithName_stringValue("linkType","ltPageLink"));
  link.addAttribute(NSXMLNode.attributeWithName_stringValue("isApplyIncludePage","False"));
  link.addAttribute(NSXMLNode.attributeWithName_stringValue("title",""));

  var area = NSXMLNode.elementWithName("area");
  link.addChild(area);
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("shape","0"));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("alt",""));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("ID","{}"));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("coords",""));

  var interactiveData = NSXMLNode.elementWithName("interactiveData");
  controlProperties.addChild(interactiveData);
  var entry = NSXMLNode.elementWithName("entry");
  interactiveData.addChild(entry);
  entry.addAttribute(NSXMLNode.attributeWithName_stringValue("title",""));
  area = NSXMLNode.elementWithName("area");
  entry.addChild(area);
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("shape","0"));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("alt",""));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("ID","{}"));
  area.addAttribute(NSXMLNode.attributeWithName_stringValue("coords","()"));

  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("rotationAngle", "TdZero"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("clipShapeType", "SbRectangle"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("radius", "0"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("radiusIsPercent", "False"));

  var url = artboardData.items[0].filename;
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("URL", url));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("FlipHorizontal", "False"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("FlipVertical", "False"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("fitMode", "2"));
  controlProperties.addChild(NSXMLNode.elementWithName_stringValue("showText", "True"));

  return control;
}

function writeProjectProperties(folderURL) {

  var pageSize = getBoxSizeStr(projectProperties);

  var nsURLProperties = NSURL.URLWithString_relativeToURL("Main.mpk",folderURL);
  var xmlDoc = NSXMLDocument.alloc().initWithContentsOfURL_options_error(nsURLProperties,NSXMLDocumentTidyXML,null);

  var root = xmlDoc.rootElement();
  root.attributeForName("name").setStringValue(projectProperties.projectName);
  root.attributeForName("ID").setStringValue(createNewUUID());

  var children = root.children();
  for (var i = 0; i < children.length; i++) {
    var childNode = children[i];
    if(childNode.name()=="params") {
      var createParam = NSXMLNode.elementWithName("createParam");
      childNode.addChild(createParam);
      createParam.addChild(NSXMLNode.elementWithName_stringValue("createDate", getNowFormatDate()));
      createParam.addChild(NSXMLNode.elementWithName_stringValue("from", "sketch"));

      var paramChild = childNode.children();
      for(var j=0;j<paramChild.length;j++) {
        var paramNode = paramChild[j];
        if(paramNode.name()=="projectParam") {
          paramNode.attributeForName("pageSize").setStringValue(pageSize);
          paramNode.attributeForName("platform").setStringValue(projectProperties.platform);
          if(isOrientation(selectedDeviceType)) {
            paramNode.addAttribute(NSXMLNode.attributeWithName_stringValue("orientation",projectProperties.orientation));
            paramNode.addAttribute(NSXMLNode.attributeWithName_stringValue("mobileScreenScale","1.77777779102325"));
            paramNode.addAttribute(NSXMLNode.attributeWithName_stringValue("mobileType",projectProperties.mobileType));
          }
        }
      }
    }
  }
  writeXMLDoc(xmlDoc,nsURLProperties.path());
}

function writePageToMain(folderURL,pageID,artboardData) {
  var imageData = artboardData.items[0];
  var imgKey = imageData.filename;
  var nsURLProperties = NSURL.URLWithString_relativeToURL("Main.mpk",folderURL);
  var xmlDoc = NSXMLDocument.alloc().initWithContentsOfURL_options_error(nsURLProperties,NSXMLDocumentTidyXML,null);

  var root = xmlDoc.rootElement();
  var children = root.children();
  for (var i = 0; i < children.length; i++) {
    var childNode = children[i];
    if(childNode.name()=="pages") {
      var page = NSXMLNode.elementWithName("page");
      childNode.addChild(page);
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("name",artboardData.name));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("ID",pageID));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("expanded","True"));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("code",""));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("thumbCode",""));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("status","psWhite"));
      page.addAttribute(NSXMLNode.attributeWithName_stringValue("isfolder","False"));

      var images = NSXMLNode.elementWithName("images");
      page.addChild(images);
      images.addChild(createImageNode(imgKey));
    }
    else if(childNode.name()=="images") {
      childNode.addChild(createImageNode(imgKey));
    }
  }
  writeXMLDoc(xmlDoc,nsURLProperties.path());
}

function createImageNode(imgKey) {
  var image = NSXMLNode.elementWithName("image");
  image.addAttribute(NSXMLNode.attributeWithName_stringValue("key",imgKey));
  return image;
}