
function getSelectedArtboards(selection){
  var skArtboards = [];
  if([selection count]>0){
    for (var i=0; i<[selection count]; i++) {
      var skLayer = [selection objectAtIndex:i];
      if(isArtboard(skLayer))
        skArtboards.push(skLayer);
    });
  }
  return skArtboards;
}

function getAllArtboards(sketchDoc){
  var skArtboards = [];
  var skPages = sketchDoc.pages;
  var pagesCount = skPages.length;
  for(var i=0;i<pagesCount;i++){
    var skPage=skPages[i];
    skPage.iterate(function(skLayer){
      if(skLayer.isArtboard){
        skArtboards.push(skLayer);
      }
    });
  }
  return skArtboards;
}

function getArtboardData(skArtboard){
  var wSkArtboard = skArtboard.sketchObject;
  var items = [];
  
  var itemData = getImageData(wSkArtboard,wSkArtboard);
  items.push(itemData);

  var absolutePosition = calculateAbsolutePosition(wSkArtboard);
  translateChildrenToRelativePosition(items,absolutePosition);
  var background;
  if(wSkArtboard.hasBackgroundColor())
    background = getMockplusColorFromNSColor(wSkArtboard.backgroundColor());
  log(skArtboard.id);
  //log(wSkArtboard.id);
  return {
    'name': skArtboard.name,
    'id':  "id",
    'background': background,
    'width': skArtboard.frame.width+"",
    'height': skArtboard.frame.height+"",
    'items': items
  };
}

function translateChildrenToRelativePosition(items,parentAbsolute){
  for (var i=0;i<items.length;i++){
    var item=items[i];
    var relativePosition = item.relativePosition;
    var absolutePosition = item.absolutePosition;
    relativePosition.x = absolutePosition.x - parentAbsolute.x;
    relativePosition.y = absolutePosition.y - parentAbsolute.y;
    if(item.type=="group"){
      translateChildrenToRelativePosition(item.items,absolutePosition);
    }
  }
}

function isSymbolMaster(wSkLayer){
  return [wSkLayer isMemberOfClass:[MSSymbolMaster class]];
}

function isSymbol(wSkLayer){
  return [wSkLayer isMemberOfClass:[MSSymbolInstance class]];
}

function isText(wSkLayer){
  return [wSkLayer isMemberOfClass:[MSTextLayer class]];
}

function isArtboard(wSkLayer){
  return [wSkLayer isMemberOfClass:[MSImmutableArtboardGroup class]] || [wSkLayer isMemberOfClass:[MSArtboardGroup class]];
}

function calculateAbsolutePosition(skLayer){
  if(isSymbolMaster(skLayer) || isSymbol(skLayer)){
    var absoluteRect = skLayer.absoluteRect();
    return {'x':absoluteRect.x(),'y':absoluteRect.y()};
  }
  if(isText(skLayer) && !enableExportTextAsImage){
    var absoluteRect = skLayer.absoluteRect();
    var angle = skLayer.rotation();
    var absolutePos = {'x':absoluteRect.x(),'y':absoluteRect.y()};
    if(angle!=0){
      var rotatedBBox = getRotatedBBoxPosition(absoluteRect,skLayer.frame(), angle);
      absolutePos.x+= absoluteRect.x() - rotatedBBox.x;
      absolutePos.y+= absoluteRect.y() - rotatedBBox.y;
    }
    return absolutePos;
  }
  var ancestry = MSImmutableLayerAncestry.ancestryWithMSLayer(skLayer);
  var rect = MSSliceTrimming.trimmedRectForLayerAncestry(ancestry);
  return {'x':rect.origin.x,'y':rect.origin.y};
}

function getMockplusColorFromNSColor(nsColor){
  return getMockplusColor(nsColor.red(),nsColor.green(),nsColor.blue(), nsColor.alpha());
}

function getMockplusColor(red,green,blue,alpha){
  var r = Math.round(red*255) * Math.pow(2,16);
  var g = Math.round(green*255) * Math.pow(2,8);
  var b = Math.round(blue*255);
  var a = Math.round(alpha*255) * Math.pow(2,24);
  var color = parseInt(r+g+b,10);
  return color+"";
}

function getImageData(skImage,skParent){
  var pngFile = exportLayerAsPNG(skImage);

  var ancestry = MSImmutableLayerAncestry.ancestryWithMSLayer(skImage);
  var rect = MSSliceTrimming.trimmedRectForLayerAncestry(ancestry);

  return {
    'type':"image",
    'name':skImage.name(),
    'absolutePosition':calculateAbsolutePosition(skImage),
    'relativePosition':{'x':0,'y':0},
    'size':rect.size,
    'filename':pngFile,
    'visible':skImage.isVisible()
  };
}

function exportLayerAsPNG(skLayer){
  var uuid = NSString.stringWithUUID();
  return exportLayerAsImage(skLayer,uuid,".png");
}

function exportLayerAsImage(skLayer,uuid,extension){
  var layerCopy;
  if(isArtboard(skLayer))
    layerCopy = skLayer;
  else
    layerCopy = skLayer.duplicate();

  var wSkDoc = sketchDoc.sketchObject;
  var fileFolder = getExportImageFolder();
  var outFile = fileFolder+ uuid+extension;

  var exportOptions=layerCopy.exportOptions();
  exportOptions.removeAllExportFormats();
  var exportSize = exportOptions.addExportFormat();
  if(enableExportImage2X)
    exportSize.setScale(2.0);
  else
   exportSize.setScale(1.0);

  var exportFormats = exportOptions.exportFormats();
  var slice = MSExportRequest.exportRequestsFromExportableLayer_exportFormats_useIDForName(layerCopy,exportFormats,false).firstObject();

  wSkDoc.saveExportRequest_toFile(slice, outFile);

  if(!isArtboard(skLayer))
    layerCopy.removeFromParent();
  else
    exportOptions.removeExportFormat(exportSize);

  return uuid+extension;
}

function getExportImageFolder(){
  return tempFolder.path()+"/0EDCD3EB-42DB-412B-9411-E5DD751EB4BA/";
}
