function createNewUUID(){ 
  return NSString.stringWithUUID().toUpperCase();
}

function isOrientation(type) {
  if(["Mobile","Pad"].indexOf(type) !== -1) {
    return true;
  }
  return false;
}

function isPortrait() {
  if(getDefault("exportOrientation", "Portrait") == "Portrait") {
    return true;
  }
  return false;
}

function figurePageSize(sketchData) {
  var pageSizeMap = {};
  var pageMaxCounter = 0
  var targetPageSize = "";
  for (var i = 0; i < sketchData.length; i++) {
    var artboardData = sketchData[i];
    var pageSize = getBoxSizeStr(artboardData);

    if(pageSizeMap[pageSize]) {
      pageSizeMap[pageSize]++;
    } else {
      pageSizeMap[pageSize] = 1;
    }
    if(pageSizeMap[pageSize] > pageMaxCounter) {
      pageMaxCounter = pageSizeMap[pageSize];
      targetPageSize = pageSize;
    } 
  }
  return getBoxSize(targetPageSize);
}

function checkPageSize(box) {
  if(selectedDeviceType == "Blank") {
    box.width = 6000;
    box.height = 6000;
  }
  if(selectedDeviceType == "Mobile") {
    if(isPhone(box)) {
      if(isIOSPhone(box)) {
        box["mobileType"] = "mmtIOS";
      } else {
        box["mobileType"] = "mmtAndroid";
      }
    } else {
      box.width = 375;
      box.height = 667;
      box["mobileType"] = "mmtIOS";
    }
  }
  if(selectedDeviceType == "Pad") {
    if(isPad(box)) {
      if(isIOSPad(box)) {
        box["mobileType"] = "mmtPad";
      } else {
        box["mobileType"] = "mmtTablet";
      }
    } else {
      box.width = 1024;
      box.height = 768;
      box["mobileType"] = "mmtPad";
    }
  }

  if(box.width < 200) {
    box.width = 200;
  }
  if(box.width > 6000) {
    box.width = 6000;
  }
  if(box.height < 200) {
    box.height = 200;
  }
  if(box.height > 6000) {
    box.height = 6000;
  }
}

function getPropertion(box) {
  var shorter,longer;
  if(isPortrait()) {
    shorter = parseFloat(box.width);
    longer = parseFloat(box.height);
  } else {
    shorter = parseFloat(box.height);
    longer = parseFloat(box.width);
  }
  return shorter/longer;
}

function isPhone(box) {
  if(0.5 <= getPropertion(box) && getPropertion(box) <= 0.75) {
    if(200 <= Math.min(box.width,box.height) <= 1500) {
      if(300 <= Math.max(box.width,box.height) <= 2600) {
        return true;
      }
    }
  }
  return false;
}

function isPad(box) {
  if(0.3 <= getPropertion(box) && getPropertion(box) <= 0.8) {
    if(700 <= Math.min(box.width,box.height) <= 1536) {
      if(1000 <= Math.max(box.width,box.height) <= 2500) {
        return true;
      }
    }
  }
  return false;
}

function isIOSPhone(box) {
  var shorter = Math.min(box.width,box.height);
  var longer = Math.max(box.width,box.height);
  if(shorter == "375" && longer == "667") {
    return true;
  }
  if(shorter == "414" && longer == "736") {
    return true;
  }
  if(shorter == "320" && longer == "568") {
    return true;
  }
  return false;
}

function isIOSPad(box) {
  var shorter = Math.min(box.width,box.height);
  var longer = Math.max(box.width,box.height);
  if(shorter == "768" && longer == "1024") {
    return true;
  }
  if(shorter == "1024" && longer == "1366") {
    return true;
  }
  return false;
}

function getBoxSizeStr(boxSize) {
  return "(" + boxSize.width + "," + boxSize.height + ")";
}

function getBoxSize(boxSizeStr) {
  var str = boxSizeStr.substring(1, boxSizeStr.length-1);
  var arr = str.split(",");
  return {width: arr[0],height:arr[1]};
}

function getProjectName(jimfile) {
  var projectName=jimfile.replace(/(.*\/)*([^.]+).*/ig,"$2");
  return projectName;
}

function getNowFormatDate() {
    var date = new Date();
    var seperator1 = "-";
    var seperator2 = ":";
    var month = date.getMonth() + 1;
    var strDate = date.getDate();
    if (month >= 1 && month <= 9) {
        month = "0" + month;
    }
    if (strDate >= 0 && strDate <= 9) {
        strDate = "0" + strDate;
    }
    var currentdate = date.getFullYear() + seperator1 + month + seperator1 + strDate
            + " " + date.getHours() + seperator2 + date.getMinutes()
            + seperator2 + date.getSeconds();
    return currentdate;
}

function getLocalVersion(context) {
    var manifestPath = context.scriptPath.stringByDeletingLastPathComponent() + '/manifest.json';
    var manifest = NSData.dataWithContentsOfFile(manifestPath);
    var data = NSJSONSerialization.JSONObjectWithData_options_error(manifest, 0, null);
    return String(data.version).toString();
}

function semverCompare(versionA, versionB) {
    var versionSegmentsA = versionA.split('.');
    var versionSegmentsB = versionB.split('.');
    for (var i = 0; i < 3; i++) {
        var versionSegmentA = parseInt(versionSegmentsA[i], 10);
        var versionSegmentB = parseInt(versionSegmentsB[i], 10);
        if (versionSegmentA > versionSegmentB) return true;
        if (versionSegmentB > versionSegmentA) return false;
        if (!isNaN(versionSegmentA) && isNaN(versionSegmentB)) return true;
        if (isNaN(versionSegmentA) && !isNaN(versionSegmentB)) return false;
    }
    return false;
 }

function getDefault(key,initialValue) {
  var defaults = [NSUserDefaults standardUserDefaults];
  var defaultValue = [defaults objectForKey: '-' + kPluginDomain + '-' + key];
  if (defaultValue != nil && ([defaultValue class] === NSDictionary)) return [NSMutableDictionary dictionaryWithDictionary:defaultValue]
  if (defaultValue != nil)
    return defaultValue;
  return initialValue;
}

function setDefault(key, value) {
  var defaults = [NSUserDefaults standardUserDefaults];
  var	configs  = [NSMutableDictionary dictionary];
  [configs setObject: value forKey: '-' + kPluginDomain + '-' + key];
  return [defaults registerDefaults: configs];
}

function _getObjectForKey(key) {
  return NSUserDefaults.standardUserDefaults().objectForKey(key);
}

function _getValueForKey(key) {
  return NSUserDefaults.standardUserDefaults().stringForKey(key);
}

function _setValueForKey(value, key) {
  NSUserDefaults.standardUserDefaults().setObject_forKey(value, key);
  NSUserDefaults.standardUserDefaults().synchronize();
}

function _removeValueForKey(key) {
  NSUserDefaults.standardUserDefaults().removeObjectForKey(key);
  NSUserDefaults.standardUserDefaults().synchronize();
}