
@import 'common.js'
@import 'sketch.js'
@import 'mockplus.js'
@import 'fileUtils.js'
@import 'dialogs.js'
@import 'api.js'

function isNeedLogin() {
	var sid = _getValueForKey("sid");
	var responseData = validSid(sid);
	if(responseData.code == "401") {
	    return true;
	} else {
		return false;
	}
}


var updateWindowManager = {

	windowContext: {},

	launch: function() {
		var responseData = getLastVersion(this.windowContext.localVersion);
	    var skipedVersion = getDefault("skipedVersion", "0");
	  
	    if(responseData.code === 0 && responseData.value.version != skipedVersion) {
	      setDefault("skipedVersion", responseData.value.version);
	      openUpdatePluginDialog(responseData, this); 
	    } else {
	      this.launchNextWindow();
	    }
	},

	callback: function(result) {
		if(result.option === "update") {
		    router.externalLink("update", result.extra.downloadURL);
		}
		if(result.option === "ignore") {
			this.launchNextWindow();
		}
	},

	launchNextWindow: function() {
		var option = router.applicationContext.command;
		log(option);
		if(option === "upload") {
			//var loginWindowContext = router.getWindowContext("login");
		    //loginWindowContext.nextWindowName = "upload";
			//router.launchWindow("login");
			router.launchWindow("upload");
		}
	    if(option === "export") {
			router.launchWindow("exportMp");
		}
	}
}

var loginWindowManager = {
	windowContext: {},

	launch: function() {
		openLoginDialog(this);
		//var sid = _getValueForKey("sid");
		//var responseData = validSid(sid);
		//this.launchNextWindow();
		if(isLogged()) {
		    //this.launchNextWindow();
		} else {
			log("open login dialog");
		    //openLoginDialog(this);
		}
	},

	shutDownCallback: function(option) {
		log("shut down option");
		if(option === "regist") {
			var registWindowContext = router.getWindowContext("regist");
		    registWindowContext.nextWindowName = this.windowContext.nextWindowName;
		    router.launchWindow("regist");
		}
		if(option === "loginSuccess") {
			this.launchNextWindow(); 
		}
		if(option === "forget") {
			router.externalLink("forget");
		}
	},

	launchNextWindow: function() {
		var windowName = this.windowContext.nextWindowName;
		if(windowName) {
			router.launchWindow(windowName);
		}
	},

	handleLoginRequest: function(userInfo, callback) {
		log("handle login");
		var basicLoginDictionary = NSDictionary.alloc().initWithObjectsAndKeys(
			userInfo.email, "email", userInfo.password, "password"
		);
	    var mutableLoginDictionary = basicLoginDictionary.mutableCopy();
		var payload = NSJSONSerialization.dataWithJSONObject_options_error(mutableLoginDictionary, NSUTF8StringEncoding, null);
		var responseData = login(payload);

		var res = {errMsg: null};
		if(responseData.code === 0) {
			//保存token
			log("save:"+responseData.sign);
			
			_setValueForKey(responseData.sid, "sid");
			_setValueForKey(responseData.user.license.version, "version");
		} else {
			res.errMsg = responseData.message;
		}
		callback(res);
	}
}

var registWindowManager = {
	windowContext: {},

	launch: function() {
		log("hello");
		openRegistDialog(this);
	},

	shutDownCallback: function(option) {
		if(option === "login") {
		    //router.externalLink("login");
		    router.launchWindow("login");
		}
		if(option === "registSuccess") {
			this.launchNextWindow();
		}
	},

	launchNextWindow: function() {
		var windowName = this.windowContext.nextWindowName
		if(windowName) {
			router.launchWindow(windowName);
		}
	},

	handleRegistRequest: function(userInfo, callback) {
		var basicLoginDictionary = NSDictionary.alloc().initWithObjectsAndKeys(
			userInfo.email, "email", 
			userInfo.password, "password"
		);
	    var mutableRegistDictionary = basicLoginDictionary.mutableCopy();
		var payload = NSJSONSerialization.dataWithJSONObject_options_error(mutableRegistDictionary, NSUTF8StringEncoding, null);
		var responseData = regist(payload);

		var res = {errMsg: null};
		if(responseData.code === 0) {
			//保存token
			log("save:"+responseData.sign);
			//_setValueForKey(responseData.sign, "sign");
			_setValueForKey(responseData.sid, "sid");
			//setDefault("sign", responseData.sign);
		} else {
			res.errMsg = responseData.message;
		}
		callback(res);
	}
}

var exportWindowManager = {
	windowContext: {},

	launch: function() {
		if(artboardsToExport.length==0){
		    alert(text.unfind_artboard_title, text.unfind_artboard_content);
		    return;
		}
		openExportDialog(this.callback);
	},

	callback: function(result) {
		if(result.option === "export") {
		    router.launchWindow("doExport");
		}
	}
}

var uploadWindowManager = {
	windowContext: {},

	launch: function() {
		if(artboardsToExport.length==0){
		    alert(text.unfind_artboard_title, text.unfind_artboard_content);
		    return;
		}
		openUploadDialog(this.callback);
	},

	callback: function(result) {
		log(result.option);
		if(result.option === "upload") {
			router.launchWindow("doUpload");
		}
		if(result.option === "buy") {

			var url = text.base_url + "redirect/upgrade?sid="+_getValueForKey("sid");
			router.externalLink("buy", url);
		}
		if(result.option === "login") {
			var context = router.getWindowContext("login");
			context.nextWindowName = "upload";
			router.launchWindow("login");
		}
	}
}

var uploadProcessWindowManager = {
	windowContext: {},

	launch: function() {
		if(this.windowContext.state == "publish") {
			openPublishDialog(this);
		} else {
			var name = nsDoc.displayName().replace(".sketch", "") + ".mp";
			var filePath = tempFolder.path() + "/" + name; 
			if(filePath){
				log(filePath);
			    setDefault("projectName",getProjectName(filePath));
			    openUploadProgressDialog(filePath, this);
			}
		}
		
	},

	callback: function(option) {
		if(option === "viewProject") {
			log(this.windowContext.url);
		    router.externalLink("default", this.windowContext.url);
		}
		if(option === "login") {
			this.windowContext.state = "publish";
			var context = router.getWindowContext("login");
			context.nextWindowName = "doUpload";
			router.launchWindow("login");
		}
		if(option === "upload") {
		    createMockplusXMLData(tempFolder,this.windowContext.data);
		    zipMpFile(tempFolder,this.windowContext.path);
		    var responseData = upload(this.windowContext.path);
		    if(responseData.code == 0) {
		    	this.windowContext.url = responseData.URL;
		    	if(!responseData.URL) {
		    		responseData.code = -1;
		    		responseData.message = "发布失败";
		    	}
		    }
		    return responseData;
		}
	}
}

var exportProcessWindowManager = {
	windowContext: {},

	launch: function() {
		var filePath = openSaveFileDialog();
		if(filePath){
		    setDefault("projectName",getProjectName(filePath));
		    openProgressDialog(filePath, this.callback);
		}
	},

	callback: function(result) {
		if(result.option === "openFile") {	
		    openMpFile(tempFolder, result.extra.path);
		    removeFolder(tempFolder);
		}
		if(result.option === "saveFile") {
		    createMockplusXMLData(tempFolder,result.extra.data);
		    zipMpFile(tempFolder,result.extra.path);
		}
	}
}

var router = {
	applicationContext: {},
	routeTable: {
		checkUpdate: updateWindowManager,
		login: loginWindowManager,
		regist: registWindowManager,
		exportMp: exportWindowManager,
		upload: uploadWindowManager,
		doUpload: uploadProcessWindowManager,
		doExport: exportProcessWindowManager
	},

	getWindowManager: function(windowKey) {
		if(windowKey === undefined || !this.routeTable[windowKey]) {
			return;
		}
		return this.routeTable[windowKey];
	},

	getWindowContext: function(windowKey) {
		var windowManager = this.getWindowManager(windowKey);
		if(windowManager) {
			return windowManager.windowContext;
		}
		return {};
	},

	launchWindow: function(windowKey) {
		var windowManager = this.getWindowManager(windowKey);
		windowManager.launch();
	},

	externalLink: function(linkKey, url) {
		var URL = url;
		if(!URL) {
			log(linkKey);
			switch(linkKey) {
				case "forget":
					URL = text.base_url + "user/forgetPassword";
					break;
				case "buy":
					URL = text.base_url + "buy";
					break;
				default:
					URL = text.base_url;
			}
		}
		log(URL);
		
		NSWorkspace.sharedWorkspace().openURL(NSURL.URLWithString(URL));
	}
}